// Header file
#ifndef ICD2_DEBUG
 #pragma chip PIC18F1230, core 16, code 2048, ram 0 : 0xFF

#else
 #pragma chip PIC18F1230, core 16, code 0x6E0, ram 0 : 0xFF
 // NOTE: last 288 words are reserved for ICD2 debugging

 char reservedICD2[5] @ 0xFB;  // reserved RAM for ICD2

 #pragma stackLevels 29   // reserve two levels for debugging
#endif

// Extended Instruction Set

/* Predefined:
TOSU,TOSH,TOSL,
STKPTR,
PCLATU,PCLATH,PCL,
TBLPTRU,TBLPTRH,TBLPTRL,TBLPTR,TABLAT,
PRODH,PRODL,
INTCON,INTCON2,INTCON3,
INDF0,POSTINC0,POSTDEC0,PREINC0,PLUSW0,FSR0H,FSR0L,FSR0,
WREG,
INDF1,POSTINC1,POSTDEC1,PREINC1,PLUSW1,FSR1H,FSR1L,FSR1,
BSR,BSRL,
INDF2,POSTINC2,POSTDEC2,PREINC2,PLUSW2,FSR2H,FSR2L,FSR2,
STATUS,
Carry,DC,Zero_,Overflow,Negative,
*/

char TMR0H    @ 0xFD7;
char TMR0L    @ 0xFD6;
char T0CON    @ 0xFD5;

char OSCCON   @ 0xFD3;
char LVDCON   @ 0xFD2;
char WDTCON   @ 0xFD1;
char RCON     @ 0xFD0;
char TMR1H    @ 0xFCF;
char TMR1L    @ 0xFCE;
char T1CON    @ 0xFCD;

char ADRESH   @ 0xFC4;
char ADRESL   @ 0xFC3;
char ADCON0   @ 0xFC2;
char ADCON1   @ 0xFC1;
char ADCON2   @ 0xFC0;

char BAUDCON  @ 0xFB8;

char CVRCON   @ 0xFB5;
char CMCON    @ 0xFB4;

char SPBRGH   @ 0xFB0;
char SPBRG    @ 0xFAF;
char RCREG    @ 0xFAE;
char TXREG    @ 0xFAD;
char TXSTA    @ 0xFAC;
char RCSTA    @ 0xFAB;

char EEADR    @ 0xFA9;
char EEDATA   @ 0xFA8;
char EECON2   @ 0xFA7;
char EECON1   @ 0xFA6;
char IPR3     @ 0xFA5;
char PIR3     @ 0xFA4;
char PIE3     @ 0xFA3;
char IPR2     @ 0xFA2;
char PIR2     @ 0xFA1;
char PIE2     @ 0xFA0;
char IPR1     @ 0xF9F;
char PIR1     @ 0xF9E;
char PIE1     @ 0xF9D;

char OSCTUNE  @ 0xF9B;
char PTCON0   @ 0xF9A;
char PTCON1   @ 0xF99;
char PTMRL    @ 0xF98;
char PTMRH    @ 0xF97;
char PTPERL   @ 0xF96;
char PTPERH   @ 0xF95;

char TRISB    @ 0xF93;
char TRISA    @ 0xF92;
char PDC0L    @ 0xF91;
char PDC0H    @ 0xF90;
char PDC1L    @ 0xF8F;
char PDC1H    @ 0xF8E;
char PDC2L    @ 0xF8D;
char PDC2H    @ 0xF8C;
char FLTCONFIG @ 0xF8B;
char LATB     @ 0xF8A;
char LATA     @ 0xF89;
char SEVTCMPL @ 0xF88;
char SEVTCMPH @ 0xF87;
char PWMCON0  @ 0xF86;
char PWMCON1  @ 0xF85;
char DTCON    @ 0xF84;
char OVDCOND  @ 0xF83;
char OVDCONS  @ 0xF82;
char PORTB    @ 0xF81;
char PORTA    @ 0xF80;

bit RBIF    @ INTCON.0;
bit INT0IF  @ INTCON.1;
bit TMR0IF  @ INTCON.2;
bit RBIE    @ INTCON.3;
bit INT0IE  @ INTCON.4;
bit TMR0IE  @ INTCON.5;
bit PEIE    @ INTCON.6;
bit GIEL    @ INTCON.6;
bit GIE     @ INTCON.7;
bit GIEH    @ INTCON.7;

bit RBIP    @ INTCON2.0;
bit INT3IP  @ INTCON2.1;
bit TMR0IP  @ INTCON2.2;
bit INTEDG3 @ INTCON2.3;
bit INTEDG2 @ INTCON2.4;
bit INTEDG1 @ INTCON2.5;
bit INTEDG0 @ INTCON2.6;
bit RBPU    @ INTCON2.7;

bit INT1IF  @ INTCON3.0;
bit INT2IF  @ INTCON3.1;
bit INT3IF  @ INTCON3.2;
bit INT1IE  @ INTCON3.3;
bit INT2IE  @ INTCON3.4;
bit INT3IE  @ INTCON3.5;
bit INT1IP  @ INTCON3.6;
bit INT2IP  @ INTCON3.7;

bit PSA     @ T0CON.3;
bit T0SE    @ T0CON.4;
bit T0CS    @ T0CON.5;
bit T016BIT @ T0CON.6;
bit TMR0ON  @ T0CON.7;

bit SCS0    @ OSCCON.0;
bit SCS1    @ OSCCON.1;
bit IOFS    @ OSCCON.2;
bit OSTS    @ OSCCON.3;
bit IDLEN   @ OSCCON.7;

bit LVDEN   @ LVDCON.4;
bit IRVST   @ LVDCON.5;

bit SWDTEN  @ WDTCON.0;

bit BOR_    @ RCON.0;
bit POR_    @ RCON.1;
bit PD      @ RCON.2;
bit TO      @ RCON.3;
bit RI_     @ RCON.4;
bit SBOREN  @ RCON.6;
bit IPEN    @ RCON.7;

bit TMR1ON  @ T1CON.0;
bit TMR1CS  @ T1CON.1;
bit T1SYNC_ @ T1CON.2;
bit T1OSCEN @ T1CON.3;
bit T1CKPS0 @ T1CON.4;
bit T1CKPS1 @ T1CON.5;
bit T1RUN   @ T1CON.6;
bit RD16    @ T1CON.7;

bit ADON    @ ADCON0.0;
bit GO      @ ADCON0.1;
bit CHS0    @ ADCON0.2;
bit CHS1    @ ADCON0.3;
bit SEVTEN  @ ADCON0.7;

bit VCFG0   @ ADCON1.4;

bit ADCS0   @ ADCON2.0;
bit ADCS1   @ ADCON2.1;
bit ADCS2   @ ADCON2.2;
bit ADFM    @ ADCON2.7;

bit ABDEN   @ BAUDCON.0;
bit WUE     @ BAUDCON.1;
bit BRG16   @ BAUDCON.3;
bit SCKP    @ BAUDCON.4;
bit RCIDL   @ BAUDCON.6;
bit ABDOVF  @ BAUDCON.7;

bit CVRSS   @ CVRCON.4;
bit CVRR    @ CVRCON.5;
bit CVREN   @ CVRCON.7;

bit CMEN0   @ CMCON.0;
bit CMEN1   @ CMCON.1;
bit CMEN2   @ CMCON.2;
bit C0OUT   @ CMCON.5;
bit C1OUT   @ CMCON.6;
bit C2OUT   @ CMCON.7;

bit TX9D    @ TXSTA.0;
bit TRMT    @ TXSTA.1;
bit BRGH    @ TXSTA.2;
bit SENDB   @ TXSTA.3;
bit SYNC    @ TXSTA.4;
bit TXEN    @ TXSTA.5;
bit TX9     @ TXSTA.6;
bit CSRC    @ TXSTA.7;

bit RX9D    @ RCSTA.0;
bit OERR    @ RCSTA.1;
bit FERR    @ RCSTA.2;
bit ADDEN   @ RCSTA.3;
bit CREN    @ RCSTA.4;
bit SREN    @ RCSTA.5;
bit RX9     @ RCSTA.6;
bit SPEN    @ RCSTA.7;

bit RD      @ EECON1.0;
bit WR      @ EECON1.1;
bit WREN    @ EECON1.2;
bit WRERR   @ EECON1.3;
bit FREE    @ EECON1.4;
bit CFGS    @ EECON1.6;
bit EEPGD   @ EECON1.7;

bit PTIP    @ IPR3.4;

bit PTIF    @ PIR3.4;

bit PTIE    @ PIE3.4;

bit LVDIP   @ IPR2.2;
bit EEIP    @ IPR2.4;
bit OSCFIP  @ IPR2.7;

bit LVDIF   @ PIR2.2;
bit EEIF    @ PIR2.4;
bit OSCFIF  @ PIR2.7;

bit LVDIE   @ PIE2.2;
bit EEIE    @ PIE2.4;
bit OSCFIE  @ PIE2.7;

bit TMR1IP  @ IPR1.0;
bit CMP0IP  @ IPR1.1;
bit CMP1IP  @ IPR1.2;
bit CMP2IP  @ IPR1.3;
bit TXIP    @ IPR1.4;
bit RCIP    @ IPR1.5;
bit ADIP    @ IPR1.6;

bit TMR1IF  @ PIR1.0;
bit CMP0IF  @ PIR1.1;
bit CMP1IF  @ PIR1.2;
bit CMP2IF  @ PIR1.3;
bit TXIF    @ PIR1.4;
bit RCIF    @ PIR1.5;
bit ADIF    @ PIR1.6;

bit TMR1IE  @ PIE1.0;
bit CMP0IE  @ PIE1.1;
bit CMP1IE  @ PIE1.2;
bit CMP2IE  @ PIE1.3;
bit TXIE    @ PIE1.4;
bit RCIE    @ PIE1.5;
bit ADIE    @ PIE1.6;

bit PLLEN   @ OSCTUNE.6;
bit INTSRC  @ OSCTUNE.7;

bit PTMOD0  @ PTCON0.0;
bit PTMOD1  @ PTCON0.1;
bit PTCKPS0 @ PTCON0.2;
bit PTCKPS1 @ PTCON0.3;

bit PTDIR   @ PTCON1.6;
bit PTEN    @ PTCON1.7;

bit FLTAEN  @ FLTCONFIG.0;
bit FLTAMOD @ FLTCONFIG.1;
bit FLTAS   @ FLTCONFIG.2;
bit BRFEN   @ FLTCONFIG.7;

bit OSYNC   @ PWMCON1.0;
bit UDIS    @ PWMCON1.1;
bit SEVTDIR @ PWMCON1.3;

bit DTPS0   @ DTCON.6;
bit DTPS1   @ DTCON.7;


#if __CC8E__ >= 1500
#pragma config /1 0xF0 OSC = LP // LP Oscillator
#pragma config /1 0xF1 OSC = XT // XT Oscillator
#pragma config /1 0xF2 OSC = HS // HS Oscillator
#pragma config /1 0xF3 OSC = RC // External RC oscillator, CLKO function on RA6
#pragma config /1 0xF4 OSC = EC // EC oscillator, CLKO function on RA6
#pragma config /1 0xF5 OSC = ECIO // EC oscillator, port function on RA6
#pragma config /1 0xF6 OSC = HSPLL // HS oscillator, PLL enabled (Clock Frequency = 4 x FOSC1)
#pragma config /1 0xF7 OSC = RCIO // External RC oscillator, port function on RA6
#pragma config /1 0xF8 OSC = INTIO2 // Internal oscillator, port function on RA6 and RA7
#pragma config /1 0xF9 OSC = INTIO1 // Internal oscillator, CLKO function on RA6, port function on RA7
#pragma config /1 0xBF FCMEN = OFF // Fail-Safe Clock Monitor disabled
#pragma config /1 0xFF FCMEN = ON // Fail-Safe Clock Monitor enabled
#pragma config /1 0x7F IESO = OFF // Oscillator Switchover mode disabled
#pragma config /1 0xFF IESO = ON // Oscillator Switchover mode enabled
#pragma config /2 0xFE PWRT = ON // PWRT enabled
#pragma config /2 0xFF PWRT = OFF // PWRT disabled
#pragma config /2 0xF9 BOR = OFF // Brown-out Reset disabled in hardware and software
#pragma config /2 0xFB BOR = SBORENCTRL // Brown-out Reset enabled and controlled by software (SBOREN is enabled)
#pragma config /2 0xFD BOR = BOACTIVE // Brown-out Reset enabled in hardware only and disabled in Sleep mode (SBOREN is disabled)
#pragma config /2 0xFF BOR = BOHW // Brown-out Reset enabled in hardware only (SBOREN is disabled)
#pragma config /2 0xE7 BORV = 0 // Maximum setting
#pragma config /2 0xEF BORV = 1
#pragma config /2 0xF7 BORV = 2
#pragma config /2 0xFF BORV = 3 // Minimum setting
#pragma config /3 0xFE WDT = OFF // WDT disabled (control is placed on the SWDTEN bit)
#pragma config /3 0xFF WDT = ON // WDT enabled
#pragma config /3 0xE1 WDTPS = 1 // 1:1
#pragma config /3 0xE3 WDTPS = 2 // 1:2
#pragma config /3 0xE5 WDTPS = 4 // 1:4
#pragma config /3 0xE7 WDTPS = 8 // 1:8
#pragma config /3 0xE9 WDTPS = 16 // 1:16
#pragma config /3 0xEB WDTPS = 32 // 1:32
#pragma config /3 0xED WDTPS = 64 // 1:64
#pragma config /3 0xEF WDTPS = 128 // 1:128
#pragma config /3 0xF1 WDTPS = 256 // 1:256
#pragma config /3 0xF3 WDTPS = 512 // 1:512
#pragma config /3 0xF5 WDTPS = 1024 // 1:1024
#pragma config /3 0xF7 WDTPS = 2048 // 1:2048
#pragma config /3 0xF9 WDTPS = 4096 // 1:4096
#pragma config /3 0xFB WDTPS = 8192 // 1:8192
#pragma config /3 0xFD WDTPS = 16384 // 1:16384
#pragma config /3 0xFF WDTPS = 32768 // 1:32768
#pragma config /4 0xFD PWMPIN = ON // PWM outputs drive active states upon Reset
#pragma config /4 0xFF PWMPIN = OFF // PWM outputs disabled upon Reset
#pragma config /4 0xFB LPOL = LOW // PWM0, PWM2 and PWM4 are active-low
#pragma config /4 0xFF LPOL = HIGH // PWM0, PWM2 and PWM4 are active-high (default)
#pragma config /4 0xF7 HPOL = LOW // PWM1, PWM3 and PWM5 are active-low
#pragma config /4 0xFF HPOL = HIGH // PWM1, PWM3 and PWM5 are active-high (default)
#pragma config /5 0xFE FLTAMX = RA7 // FLTA input is muxed onto RA7
#pragma config /5 0xFF FLTAMX = RA5 // FLTA input is muxed onto RA5
#pragma config /5 0xF7 T1OSCMX = LOW // T1OSO/T1CKI pin resides on RB2
#pragma config /5 0xFF T1OSCMX = HIGH // T1OSO/T1CKI pin resides on RA6
#pragma config /5 0x7F MCLRE = OFF // RA5 input pin enabled, MCLR pin disabled
#pragma config /5 0xFF MCLRE = ON // MCLR pin enabled, RA5 input pin disabled
#pragma config /6 0xFE STVREN = OFF // Reset on stack overflow/underflow disabled
#pragma config /6 0xFF STVREN = ON // Reset on stack overflow/underflow enabled
#pragma config /6 0xCF BBSIZ = BB256 // 256 Words (512 Bytes) Boot Block size
#pragma config /6 0xFF BBSIZ = BB512 // 512 Words (1024 Bytes) Boot Block size
#pragma config /6 0xBF XINST = OFF // Instruction set extension and Indexed Addressing mode disabled
#pragma config /6 0xFF XINST = ON // Instruction set extension and Indexed Addressing mode enabled
#pragma config /6 0x7F DEBUG = ON // Background debugger enabled, RB6 and RB7 are dedicated to In-Circuit Debug
#pragma config /6 0xFF DEBUG = OFF // Background debugger disabled, RB6 and RB7 configured as general purpose I/O pins
#pragma config /8 0xFE CP0 = ON // Block 0 is code-protected
#pragma config /8 0xFF CP0 = OFF // Block 0 is not code-protected
#pragma config /8 0xFD CP1 = ON // Block 1 is code-protected
#pragma config /8 0xFF CP1 = OFF // Block 1 is not code-protected
#pragma config /9 0xBF CPB = ON // Boot Block is code-protected
#pragma config /9 0xFF CPB = OFF // Boot Block is not code-protected
#pragma config /9 0x7F CPD = ON // Data EEPROM is code-protected
#pragma config /9 0xFF CPD = OFF // Data EEPROM is not code-protected
#pragma config /10 0xFE WRT0 = ON // Block 0 is write-protected
#pragma config /10 0xFF WRT0 = OFF // Block 0 is not write-protected
#pragma config /10 0xFD WRT1 = ON // Block 1 is write-protected
#pragma config /10 0xFF WRT1 = OFF // Block 1 is not write-protected
#pragma config /11 0xDF WRTC = ON // Configuration registers are write-protected
#pragma config /11 0xFF WRTC = OFF // Configuration registers are not write-protected
#pragma config /11 0xBF WRTB = ON // Boot Block is write-protected
#pragma config /11 0xFF WRTB = OFF // Boot Block is not write-protected
#pragma config /11 0x7F WRTD = ON // Data EEPROM is write-protected
#pragma config /11 0xFF WRTD = OFF // Data EEPROM is not write-protected
#pragma config /12 0xFE EBTR0 = ON // Block 0 is protected from table reads executed in other blocks
#pragma config /12 0xFF EBTR0 = OFF // Block 0 is not protected from table reads executed in other blocks
#pragma config /12 0xFD EBTR1 = ON // Block 1 is protected from table reads executed in other blocks
#pragma config /12 0xFF EBTR1 = OFF // Block 1 is not protected from table reads executed in other blocks
#pragma config /13 0xBF EBTRB = ON // Boot Block is protected from table reads executed in other blocks
#pragma config /13 0xFF EBTRB = OFF // Boot Block is not protected from table reads executed in other blocks
#endif
