// Header file
#ifndef ICD2_DEBUG
 #pragma chip PIC18F2450, core 16, code 8192, ram 0 : 0x4FF

#else
 #pragma chip PIC18F2450, core 16, code 0x1EE0, ram 0 : 0x4FF
 // NOTE: last 288 words are reserved for ICD2 debugging

 char reservedICD2[12] @ 0x1F4;  // reserved RAM for ICD2

 #pragma stackLevels 29   // reserve two levels for debugging

#endif

char unusedRA[512] @ 0x200;

// Extended Instruction Set

#pragma accessGPR 0x60

/* Predefined:
TOSU,TOSH,TOSL,
STKPTR,
PCLATU,PCLATH,PCL,
TBLPTRU,TBLPTRH,TBLPTRL,TBLPTR,TABLAT,
PRODH,PRODL,
INTCON,INTCON2,INTCON3,
INDF0,POSTINC0,POSTDEC0,PREINC0,PLUSW0,FSR0H,FSR0L,FSR0,
WREG,
INDF1,POSTINC1,POSTDEC1,PREINC1,PLUSW1,FSR1H,FSR1L,FSR1,
BSR,BSRL,
INDF2,POSTINC2,POSTDEC2,PREINC2,PLUSW2,FSR2H,FSR2L,FSR2,
STATUS,
Carry,DC,Zero_,Overflow,Negative,
*/

char TMR0H    @ 0xFD7;
char TMR0L    @ 0xFD6;
char T0CON    @ 0xFD5;

char OSCCON   @ 0xFD3;
char HLVDCON  @ 0xFD2;
char WDTCON   @ 0xFD1;
char RCON     @ 0xFD0;
char TMR1H    @ 0xFCF;
char TMR1L    @ 0xFCE;
char T1CON    @ 0xFCD;
char TMR2     @ 0xFCC;
char PR2      @ 0xFCB;
char T2CON    @ 0xFCA;

char ADRESH   @ 0xFC4;
char ADRESL   @ 0xFC3;
char ADCON0   @ 0xFC2;
char ADCON1   @ 0xFC1;
char ADCON2   @ 0xFC0;
char CCPR1H   @ 0xFBF;
char CCPR1L   @ 0xFBE;
char CCP1CON  @ 0xFBD;

char BAUDCON  @ 0xFB8;

char SPBRGH   @ 0xFB0;
char SPBRG    @ 0xFAF;
char RCREG    @ 0xFAE;
char TXREG    @ 0xFAD;
char TXSTA    @ 0xFAC;
char RCSTA    @ 0xFAB;

char EECON2   @ 0xFA7;
char EECON1   @ 0xFA6;

char IPR2     @ 0xFA2;
char PIR2     @ 0xFA1;
char PIE2     @ 0xFA0;
char IPR1     @ 0xF9F;
char PIR1     @ 0xF9E;
char PIE1     @ 0xF9D;

char TRISC    @ 0xF94;
char TRISB    @ 0xF93;
char TRISA    @ 0xF92;

char LATC     @ 0xF8B;
char LATB     @ 0xF8A;
char LATA     @ 0xF89;

char PORTE    @ 0xF84;

char PORTC    @ 0xF82;
char PORTB    @ 0xF81;
char PORTA    @ 0xF80;
char UEP15    @ 0xF7F;
char UEP14    @ 0xF7E;
char UEP13    @ 0xF7D;
char UEP12    @ 0xF7C;
char UEP11    @ 0xF7B;
char UEP10    @ 0xF7A;
char UEP9     @ 0xF79;
char UEP8     @ 0xF78;
char UEP7     @ 0xF77;
char UEP6     @ 0xF76;
char UEP5     @ 0xF75;
char UEP4     @ 0xF74;
char UEP3     @ 0xF73;
char UEP2     @ 0xF72;
char UEP1     @ 0xF71;
char UEP0     @ 0xF70;
char UCFG     @ 0xF6F;
char UADDR    @ 0xF6E;
char UCON     @ 0xF6D;
char USTAT    @ 0xF6C;
char UEIE     @ 0xF6B;
char UEIR     @ 0xF6A;
char UIE      @ 0xF69;
char UIR      @ 0xF68;
char UFRMH    @ 0xF67;
char UFRML    @ 0xF66;

bit RBIF    @ INTCON.0;
bit INT0IF  @ INTCON.1;
bit TMR0IF  @ INTCON.2;
bit RBIE    @ INTCON.3;
bit INT0IE  @ INTCON.4;
bit TMR0IE  @ INTCON.5;
bit PEIE    @ INTCON.6;
bit GIEL    @ INTCON.6;
bit GIE     @ INTCON.7;
bit GIEH    @ INTCON.7;

bit RBIP    @ INTCON2.0;
bit TMR0IP  @ INTCON2.2;
bit INTEDG2 @ INTCON2.4;
bit INTEDG1 @ INTCON2.5;
bit INTEDG0 @ INTCON2.6;
bit RBPU    @ INTCON2.7;

bit INT1IF  @ INTCON3.0;
bit INT2IF  @ INTCON3.1;
bit INT1IE  @ INTCON3.3;
bit INT2IE  @ INTCON3.4;
bit INT1IP  @ INTCON3.6;
bit INT2IP  @ INTCON3.7;

bit PSA     @ T0CON.3;
bit T0SE    @ T0CON.4;
bit T0CS    @ T0CON.5;
bit T08BIT  @ T0CON.6;
bit TMR0ON  @ T0CON.7;

bit SCS0    @ OSCCON.0;
bit SCS1    @ OSCCON.1;
bit OSTS    @ OSCCON.3;
bit IDLEN   @ OSCCON.7;

bit HLVDEN  @ HLVDCON.4;
bit IRVST   @ HLVDCON.5;
bit VDIRMAG @ HLVDCON.7;

bit SWDTEN  @ WDTCON.0;

bit BOR_    @ RCON.0;
bit POR_    @ RCON.1;
bit PD      @ RCON.2;
bit TO      @ RCON.3;
bit RI_     @ RCON.4;
bit SBOREN  @ RCON.6;
bit IPEN    @ RCON.7;

bit TMR1ON  @ T1CON.0;
bit TMR1CS  @ T1CON.1;
bit T1SYNC_ @ T1CON.2;
bit T1OSCEN @ T1CON.3;
bit T1CKPS0 @ T1CON.4;
bit T1CKPS1 @ T1CON.5;
bit T1RUN   @ T1CON.6;
bit RD16    @ T1CON.7;

bit T2CKPS0 @ T2CON.0;
bit T2CKPS1 @ T2CON.1;
bit TMR2ON  @ T2CON.2;

bit ADON    @ ADCON0.0;
bit GO      @ ADCON0.1;
bit CHS0    @ ADCON0.2;
bit CHS1    @ ADCON0.3;
bit CHS2    @ ADCON0.4;
bit CHS3    @ ADCON0.5;

bit VCFG0   @ ADCON1.4;
bit VCFG1   @ ADCON1.5;

bit ADCS0   @ ADCON2.0;
bit ADCS1   @ ADCON2.1;
bit ADCS2   @ ADCON2.2;
bit ADFM    @ ADCON2.7;

bit DC1B0   @ CCP1CON.4;
bit DC1B1   @ CCP1CON.5;

bit ABDEN   @ BAUDCON.0;
bit WUE     @ BAUDCON.1;
bit BRG16   @ BAUDCON.3;
bit SCKP    @ BAUDCON.4;
bit RCIDL   @ BAUDCON.6;
bit ABDOVF  @ BAUDCON.7;

bit TX9D    @ TXSTA.0;
bit TRMT    @ TXSTA.1;
bit BRGH    @ TXSTA.2;
bit SENDB   @ TXSTA.3;
bit SYNC    @ TXSTA.4;
bit TXEN    @ TXSTA.5;
bit TX9     @ TXSTA.6;
bit CSRC    @ TXSTA.7;

bit RX9D    @ RCSTA.0;
bit OERR    @ RCSTA.1;
bit FERR    @ RCSTA.2;
bit ADDEN   @ RCSTA.3;
bit CREN    @ RCSTA.4;
bit SREN    @ RCSTA.5;
bit RX9     @ RCSTA.6;
bit SPEN    @ RCSTA.7;

bit WR      @ EECON1.1;
bit WREN    @ EECON1.2;
bit WRERR   @ EECON1.3;
bit FREE    @ EECON1.4;
bit CFGS    @ EECON1.6;

bit HLVDIP  @ IPR2.2;
bit USBIP   @ IPR2.5;
bit OSCFIP  @ IPR2.7;

bit HLVDIF  @ PIR2.2;
bit USBIF   @ PIR2.5;
bit OSCFIF  @ PIR2.7;

bit HLVDIE  @ PIE2.2;
bit USBIE   @ PIE2.5;
bit OSCFIE  @ PIE2.7;

bit TMR1IP  @ IPR1.0;
bit TMR2IP  @ IPR1.1;
bit CCP1IP  @ IPR1.2;
bit TXIP    @ IPR1.4;
bit RCIP    @ IPR1.5;
bit ADIP    @ IPR1.6;

bit TMR1IF  @ PIR1.0;
bit TMR2IF  @ PIR1.1;
bit CCP1IF  @ PIR1.2;
bit TXIF    @ PIR1.4;
bit RCIF    @ PIR1.5;
bit ADIF    @ PIR1.6;

bit TMR1IE  @ PIE1.0;
bit TMR2IE  @ PIE1.1;
bit CCP1IE  @ PIE1.2;
bit TXIE    @ PIE1.4;
bit RCIE    @ PIE1.5;
bit ADIE    @ PIE1.6;

bit PPB0    @ UCFG.0;
bit PPB1    @ UCFG.1;
bit FSEN    @ UCFG.2;
bit UTRDIS  @ UCFG.3;
bit UPUEN   @ UCFG.4;
bit UOEMON  @ UCFG.6;
bit UTEYE   @ UCFG.7;

bit SUSPND  @ UCON.1;
bit RESUME  @ UCON.2;
bit USBEN   @ UCON.3;
bit PKTDIS  @ UCON.4;
bit SE0     @ UCON.5;
bit PPBRST  @ UCON.6;

bit PPBI    @ USTAT.1;
bit DIR     @ USTAT.2;
bit ENDP0   @ USTAT.3;
bit ENDP1   @ USTAT.4;
bit ENDP2   @ USTAT.5;
bit ENDP3   @ USTAT.6;

bit PIDEE   @ UEIE.0;
bit CRC5EE  @ UEIE.1;
bit CRC16EE @ UEIE.2;
bit DFN8EE  @ UEIE.3;
bit BTOEE   @ UEIE.4;
bit BTSEE   @ UEIE.7;

bit PIDEF   @ UEIR.0;
bit CRC5EF  @ UEIR.1;
bit CRC16EF @ UEIR.2;
bit DFN8EF  @ UEIR.3;
bit BTOEF   @ UEIR.4;
bit BTSEF   @ UEIR.7;

bit URSTIE  @ UIE.0;
bit UERRIE  @ UIE.1;
bit ACTVIE  @ UIE.2;
bit TRNIE   @ UIE.3;
bit IDLEIE  @ UIE.4;
bit STALLIE @ UIE.5;
bit SOFIE   @ UIE.6;

bit URSTIF  @ UIR.0;
bit UERRIF  @ UIR.1;
bit ACTVIF  @ UIR.2;
bit TRNIF   @ UIR.3;
bit IDLEIF  @ UIR.4;
bit STALLIF @ UIR.5;
bit SOFIF   @ UIR.6;


#if __CC8E__ >= 1500
#pragma config /0 0xF8 PLLDIV = 1 // No prescale (4 MHz oscillator input drives PLL directly)
#pragma config /0 0xF9 PLLDIV = 2 // Divide by 2 (8 MHz oscillator input)
#pragma config /0 0xFA PLLDIV = 3 // Divide by 3 (12 MHz oscillator input)
#pragma config /0 0xFB PLLDIV = 4 // Divide by 4 (16 MHz oscillator input)
#pragma config /0 0xFC PLLDIV = 5 // Divide by 5 (20 MHz oscillator input)
#pragma config /0 0xFD PLLDIV = 6 // Divide by 6 (24 MHz oscillator input)
#pragma config /0 0xFE PLLDIV = 10 // Divide by 10 (40 MHz oscillator input)
#pragma config /0 0xFF PLLDIV = 12 // Divide by 12 (48 MHz oscillator input)
#pragma config /0 0xE7 CPUDIV = OSC1_PLL2 // [Primary Oscillator Src: /1][96 MHz PLL Src: /2]
#pragma config /0 0xEF CPUDIV = OSC2_PLL3 // [Primary Oscillator Src: /2][96 MHz PLL Src: /3]
#pragma config /0 0xF7 CPUDIV = OSC3_PLL4 // [Primary Oscillator Src: /3][96 MHz PLL Src: /4]
#pragma config /0 0xFF CPUDIV = OSC4_PLL6 // [Primary Oscillator Src: /4][96 MHz PLL Src: /6]
#pragma config /0 0xDF USBDIV = 1 // USB clock source comes directly from the primary oscillator block with no postscale
#pragma config /0 0xFF USBDIV = 2 // USB clock source comes from the 96 MHz PLL divided by 2
#pragma config /1 0xF0 FOSC = XT_XT // XT oscillator (XT)
#pragma config /1 0xF2 FOSC = XTPLL_XT // XT oscillator, PLL enabled (XTPLL)
#pragma config /1 0xF4 FOSC = ECIO_EC // EC oscillator, port function on RA6 (ECIO)
#pragma config /1 0xF5 FOSC = EC_EC // EC oscillator, CLKO function on RA6 (EC)
#pragma config /1 0xF6 FOSC = ECPLLIO_EC // EC oscillator, PLL enabled, port function on RA6 (ECPIO)
#pragma config /1 0xF7 FOSC = ECPLL_EC // EC oscillator, PLL enabled, CLKO function on RA6 (ECPLL)
#pragma config /1 0xF8 FOSC = INTOSCIO_EC // Internal oscillator, port function on RA6, EC used by USB (INTIO)
#pragma config /1 0xF9 FOSC = INTOSC_EC // Internal oscillator, CLKO function on RA6, EC used by USB (INTCKO)
#pragma config /1 0xFA FOSC = INTOSC_XT // Internal oscillator, XT used by USB (INTXT)
#pragma config /1 0xFB FOSC = INTOSC_HS // Internal oscillator, HS oscillator used by USB (INTHS)
#pragma config /1 0xFC FOSC = HS // HS oscillator (HS)
#pragma config /1 0xFE FOSC = HSPLL_HS // HS oscillator, PLL enabled (HSPLL)
#pragma config /1 0xBF FCMEN = OFF // Fail-Safe Clock Monitor disabled
#pragma config /1 0xFF FCMEN = ON // Fail-Safe Clock Monitor enabled
#pragma config /1 0x7F IESO = OFF // Oscillator Switchover mode disabled
#pragma config /1 0xFF IESO = ON // Oscillator Switchover mode enabled
#pragma config /2 0xFE PWRT = ON // PWRT enabled
#pragma config /2 0xFF PWRT = OFF // PWRT disabled
#pragma config /2 0xF9 BOR = OFF // Brown-out Reset disabled in hardware and software
#pragma config /2 0xFB BOR = SOFT // Brown-out Reset enabled and controlled by software (SBOREN is enabled)
#pragma config /2 0xFD BOR = ON_ACTIVE // Brown-out Reset enabled in hardware only and disabled in Sleep mode (SBOREN is disabled)
#pragma config /2 0xFF BOR = ON // Brown-out Reset enabled in hardware only (SBOREN is disabled)
#pragma config /2 0xE7 BORV = 46 // 4.6V
#pragma config /2 0xEF BORV = 43 // 4.3V
#pragma config /2 0xF7 BORV = 28 // 2.8V
#pragma config /2 0xFF BORV = 21 // 2.1V
#pragma config /2 0xDF VREGEN = OFF // USB voltage regulator disabled
#pragma config /2 0xFF VREGEN = ON // USB voltage regulator enabled
#pragma config /3 0xFE WDT = OFF // WDT disabled (control is placed on the SWDTEN bit)
#pragma config /3 0xFF WDT = ON // WDT enabled
#pragma config /3 0xE1 WDTPS = 1 // 1:1
#pragma config /3 0xE3 WDTPS = 2 // 1:2
#pragma config /3 0xE5 WDTPS = 4 // 1:4
#pragma config /3 0xE7 WDTPS = 8 // 1:8
#pragma config /3 0xE9 WDTPS = 16 // 1:16
#pragma config /3 0xEB WDTPS = 32 // 1:32
#pragma config /3 0xED WDTPS = 64 // 1:64
#pragma config /3 0xEF WDTPS = 128 // 1:128
#pragma config /3 0xF1 WDTPS = 256 // 1:256
#pragma config /3 0xF3 WDTPS = 512 // 1:512
#pragma config /3 0xF5 WDTPS = 1024 // 1:1024
#pragma config /3 0xF7 WDTPS = 2048 // 1:2048
#pragma config /3 0xF9 WDTPS = 4096 // 1:4096
#pragma config /3 0xFB WDTPS = 8192 // 1:8192
#pragma config /3 0xFD WDTPS = 16384 // 1:16384
#pragma config /3 0xFF WDTPS = 32768 // 1:32768
#pragma config /5 0xFD PBADEN = OFF // PORTB<4:0> pins are configured as digital I/O on Reset
#pragma config /5 0xFF PBADEN = ON // PORTB<4:0> pins are configured as analog input channels on Reset
#pragma config /5 0xFB LPT1OSC = OFF // Timer1 configured for higher power operation
#pragma config /5 0xFF LPT1OSC = ON // Timer1 configured for low-power operation
#pragma config /5 0x7F MCLRE = OFF // RE3 input pin enabled; MCLR pin disabled
#pragma config /5 0xFF MCLRE = ON // MCLR pin enabled; RE3 input pin disabled
#pragma config /6 0xFE STVREN = OFF // Stack full/underflow will not cause Reset
#pragma config /6 0xFF STVREN = ON // Stack full/underflow will cause Reset
#pragma config /6 0xFB LVP = OFF // Single-Supply ICSP disabled
#pragma config /6 0xFF LVP = ON // Single-Supply ICSP enabled
#pragma config /6 0xF7 BBSIZ = BB1K // 1KW Boot block size
#pragma config /6 0xFF BBSIZ = BB2K // 2KW Boot block size
#pragma config /6 0xBF XINST = OFF // Instruction set extension and Indexed Addressing mode disabled (Legacy mode)
#pragma config /6 0xFF XINST = ON // Instruction set extension and Indexed Addressing mode enabled
#pragma config /6 0x7F DEBUG = ON // Background debugger enabled, RB6 and RB7 are dedicated to In-Circuit Debug
#pragma config /6 0xFF DEBUG = OFF // Background debugger disabled, RB6 and RB7 configured as general purpose I/O pins
#pragma config /8 0xFE CP0 = ON // Block 0 (000800-001FFFh) or (001000-001FFFh) is code-protected
#pragma config /8 0xFF CP0 = OFF // Block 0 (000800-001FFFh) or (001000-001FFFh) is not code-protected
#pragma config /8 0xFD CP1 = ON // Block 1 (002000-003FFFh) is code-protected
#pragma config /8 0xFF CP1 = OFF // Block 1 (002000-003FFFh) is not code-protected
#pragma config /9 0xBF CPB = ON // Boot block (000000-0007FFh) or (000000-000FFFh) is code-protected
#pragma config /9 0xFF CPB = OFF // Boot block (000000-0007FFh) or (000000-000FFFh) is not code-protected
#pragma config /10 0xFE WRT0 = ON // Block 0 (000800-001FFFh) or (001000-001FFFh) is write-protected
#pragma config /10 0xFF WRT0 = OFF // Block 0 (000800-001FFFh) or (001000-001FFFh) is not write-protected
#pragma config /10 0xFD WRT1 = ON // Block 1 (002000-003FFFh) is write-protected
#pragma config /10 0xFF WRT1 = OFF // Block 1 (002000-003FFFh) is not write-protected
#pragma config /11 0xDF WRTC = ON // Configuration registers (300000-3000FFh) are write-protected
#pragma config /11 0xFF WRTC = OFF // Configuration registers (300000-3000FFh) are not write-protected
#pragma config /11 0xBF WRTB = ON // Boot block (000000-0007FFh) or (000000-000FFFh) is write-protected
#pragma config /11 0xFF WRTB = OFF // Boot block (000000-0007FFh) or (000000-000FFFh) is not write-protected
#pragma config /12 0xFE EBTR0 = ON // Block 0 (000800-001FFFh) or (001000-001FFFh) is protected from table reads executed in other blocks
#pragma config /12 0xFF EBTR0 = OFF // Block 0 (000800-001FFFh) or (001000-001FFFh) is not protected from table reads executed in other blocks
#pragma config /12 0xFD EBTR1 = ON // Block 1 (002000-003FFFh) is protected from table reads executed in other blocks
#pragma config /12 0xFF EBTR1 = OFF // Block 1 (002000-003FFFh) is not protected from table reads executed in other blocks
#pragma config /13 0xBF EBTRB = ON // Boot block (000000-0007FFh) or (000000-000FFFh) is protected from table reads executed in other blocks
#pragma config /13 0xFF EBTRB = OFF // Boot block (000000-0007FFh) or (000000-000FFFh) is not protected from table reads executed in other blocks
#endif
